#  musicview, (re)discover your music library.
#  Copyright (C) 2018 Peijun Ma
#
#  This program is free software: you can redistribute it and/or modify
#  it under the terms of the GNU General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  This program is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU General Public License for more details.
#
#  You should have received a copy of the GNU General Public License
#  along with this program.  If not, see <http://www.gnu.org/licenses/>.

__title__ = 'musicview'
__description__ = '(re)discover your music library.'
__url__ = 'https://github.com/MaT1g3R/musicview'
__version__ = '0.2.0'
__author__ = 'Peijun Ma'
__author_email__ = 'peijun.ma@protonmail.com'
__license__ = 'GPLv3.0+'
__copyright__ = 'Copyright (C) 2018 Peijun Ma'
